@@ -1,6 +1,6 @@
 /*
  * Copyright 2013-2014 Luke Dashjr
- * Copyright 2012-2013 Con Kolivas
+ * Copyright 2012-2014 Con Kolivas
  * Copyright 2011 Andrew Smith
  * Copyright 2011 Jeff Garzik
  *
@@ -264,6 +264,15 @@ static inline void align_len(size_t *len)
 }
 
 
+static inline
+uint8_t bitflip8(uint8_t p)
+{
+	p = ((p & 0xaa) >> 1) | ((p & 0x55) << 1);
+	p = ((p & 0xcc) >> 2) | ((p & 0x33) << 2);
+	p = ((p & 0xf0) >> 4) | ((p & 0x0f) << 4);
+	return p;
+}
+
 static inline
 uint8_t upk_u8(const void * const bufp, const int offset)
 {
@@ -271,6 +280,8 @@ uint8_t upk_u8(const void * const bufp, const int offset)
 	return buf[offset];
 }
 
+#define upk_u8be(buf, offset)  upk_u8(buf, offset)
+
 static inline
 uint16_t upk_u16be(const void * const bufp, const int offset)
 {
@@ -303,6 +314,8 @@ uint64_t upk_u64be(const void * const bufp, const int offset)
 	     | (((uint64_t)buf[offset+7]) <<    0);
 }
 
+#define upk_u8le(buf, offset)  upk_u8(buf, offset)
+
 static inline
 uint16_t upk_u16le(const void * const bufp, const int offset)
 {
@@ -343,6 +356,8 @@ void pk_u8(void * const bufp, const int offset, const uint8_t nv)
 	buf[offset] = nv;
 }
 
+#define pk_u8be(buf, offset, nv)  pk_u8(buf, offset, nv)
+
 static inline
 void pk_u16be(void * const bufp, const int offset, const uint16_t nv)
 {
@@ -375,6 +390,8 @@ void pk_u64be(void * const bufp, const int offset, const uint64_t nv)
 	buf[offset+7] = (nv >>    0) & 0xff;
 }
 
+#define pk_u8le(buf, offset, nv)  pk_u8(buf, offset, nv)
+
 static inline
 void pk_u16le(void * const bufp, const int offset, const uint16_t nv)
 {
@@ -407,6 +424,35 @@ void pk_u64le(void * const bufp, const int offset, const uint64_t nv)
 	buf[offset+7] = (nv >> 0x38) & 0xff;
 }
 
+#define _pk_uNle(bitwidth, newvalue)  do{  \
+	uint ## bitwidth ## _t _mask = 1;  \
+	_mask <<= _bitlen;  \
+	--_mask;  \
+	uint ## bitwidth ## _t _filt = _mask;  \
+	_filt <<= _bitoff;  \
+	_filt = ~_filt;  \
+	uint ## bitwidth ## _t _u = upk_u ## bitwidth ## le(_buf, 0);  \
+	_u = (_u & _filt) | (((newvalue) & _mask) << _bitoff);  \
+	pk_u ## bitwidth ## le(_buf, 0, _u);  \
+}while(0)
+
+#define pk_uNle(bufp, offset, bitoffset, bitlength, newvalue)  do{  \
+	uint8_t * const _buf = &((uint8_t *)(bufp))[offset];  \
+	const int _bitoff = (bitoffset), _bitlen = bitlength;  \
+	const int _bittot = bitoffset + bitlength;  \
+	_Static_assert((bitoffset + bitlength) <= 0x40, "Too many bits addressed in pk_uNle (bitoffset + bitlength must be <= 64)");  \
+	if (_bittot <=    8)  \
+		_pk_uNle( 8, newvalue);  \
+	else  \
+	if (_bittot <= 0x10)  \
+		_pk_uNle(16, newvalue);  \
+	else  \
+	if (_bittot <= 0x20)  \
+		_pk_uNle(32, newvalue);  \
+	else  \
+		_pk_uNle(64, newvalue);  \
+}while(0)
+
 #define is_power_of_two(n)  \
 	(0 == ((n) && ((n) - 1)))
 
@@ -767,5 +813,8 @@ extern uint8_t crc5usb(unsigned char *ptr, uint8_t len);
 extern void bfg_init_checksums(void);
 extern uint8_t crc8ccitt(const void *, size_t);
 
+extern uint16_t crc16(const void *, size_t, uint16_t init);
+#define crc16ffff(  DATA, SZ)  crc16(DATA, SZ, 0xffff)
+#define crc16xmodem(DATA, SZ)  crc16(DATA, SZ, 0)
 
 #endif /* __UTIL_H__ */
